{% extends "bootstrap/base.html" %}
{%block styles%}
{{super()}}
<link href="https://code.jquery.com/ui/1.11.3/themes/ui-lightness/jquery-ui.css"
			rel="stylesheet" media="screen">
<style>
.event.row h3 { text-align: center; }
.event.row { border-top: 1px dotted black; }
</style>
{%endblock%}
{%block scripts%}
{{super()}}
<script src="https://code.jquery.com/ui/1.11.3/jquery-ui.min.js"
				type="text/javascript">
</script>
<script type="text/javascript">
$(function() {
	$('.button').button({});
});
</script>
{%endblock%}
{% block title %}Larigira{% endblock title %}
{% block navbar %}
<nav class="navbar navbar-default">
	<div class="container-fluid">

		<div class="navbar-header">
			<button type="button" class="navbar-toggle collapsed"
												 data-toggle="collapse"
						 data-target="#bs-navbar-collapse-1">
				<span class="sr-only">Toggle navigation</span>
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
			</button>
			<a class="navbar-brand" href="{{url_for('db.home')}}">Larigira</a>
		</div>{# navbar-header #}

		<div class="collapse navbar-collapse" id="bs-navbar-collapse-1">
			<ul class="nav navbar-nav">
				<li><a href="{{url_for('db.list')}}">List</a></li>
				<li><a href="{{url_for('db.addtime')}}">Add time</a></li>
				<li><a href="{{url_for('db.addaudio')}}">Add audio</a></li>
			</ul>
		</div>{# collapse #}

	</div>{#container-fluid#}
</nav>
{% endblock %}

{% macro dict_table(obj, exclude_list=[]) %}
<table class="table table-condensed">
	<thead>
		<tr>
			<th>Key</th>
			<th>Value</th>
		</tr>
	</thead>
	<tbody>
		{% for key in obj if key not in exclude_list%}
		<tr>
			<td>{{key}}</td>
			<td>{{obj[key]}}</td>
		</tr>
		{% endfor %}
	</tbody>
</table>
{% endmacro %}

{# vim: set ts=2 sw=2 noet: #}
