{% extends "dbadmin_base.html" %}
{% import "bootstrap/wtf.html" as wtf %}

{% block title %}Larigira - DB add audio "{{kind}}" {%endblock%}

{% block content %}
{% if form.errors %}
<div class="alert alert-danger">
	<strong>There were some error in your form;</strong> please fix them, then
	resubmit
</div>
{% endif %}
<div class="container-fluid">
	{{wtf.quick_form(form)}}
</div>
{% endblock content %}
{# vim: set ts=2 sw=2 noet: #}
